package cz.zweistein.df.soundsense.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTabbedPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;

import cz.zweistein.df.soundsense.config.ConfigurationXML;
import cz.zweistein.df.soundsense.config.Sound;
import cz.zweistein.df.soundsense.config.SoundFile;
import cz.zweistein.df.soundsense.output.sound.SoundProcesor;
import cz.zweistein.df.soundsense.output.sound.player.ChannelThread;

public class Gui extends JPanel {
	private static final long serialVersionUID = 7326794456268877831L;
	
	public Gui(final ConfigurationXML config, final SoundProcesor sp) {
		setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		
        final JLabel volumeLabel = new JLabel("Volume: " + (int) config.getVolume(), JLabel.CENTER);
        volumeLabel.setAlignmentX(Component.CENTER_ALIGNMENT);

        add(volumeLabel);

        JSlider volumeSlider = new JSlider(JSlider.HORIZONTAL, -40, 6, (int) config.getVolume());
        volumeSlider.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				sp.setGlobalVolume(((JSlider) e.getSource()).getValue());
				volumeLabel.setText("volume: "+((JSlider) e.getSource()).getValue());
				config.setVolume(((JSlider) e.getSource()).getValue());
			}
		});
        
        volumeSlider.setPaintTicks(true);
        volumeSlider.setPaintLabels(true);
        volumeSlider.setMajorTickSpacing(10);
        volumeSlider.setMinorTickSpacing(5);
        
        add(volumeSlider);
        
        JTabbedPane tabPane = new JTabbedPane();
        
        add(tabPane);
        
        JPanel currentlyPlayingPanel = new JPanel();
        currentlyPlayingPanel.setLayout(new BoxLayout(currentlyPlayingPanel, BoxLayout.PAGE_AXIS));
        
        JLabel sfxListLabel = new JLabel("SFX:", JLabel.LEFT);
        currentlyPlayingPanel.add(sfxListLabel);
        
        JList sfxList = new JList(new SFXListAdapter(sp.getPlayerManager()));
        JScrollPane sfxListScroller = new JScrollPane(sfxList);
        currentlyPlayingPanel.add(sfxListScroller);
        
        JLabel chanelsListLabel = new JLabel("Channels:", JLabel.LEFT);
        currentlyPlayingPanel.add(chanelsListLabel);
        
        final JList channelList = new JList(new ChannelListAdapter(sp.getPlayerManager())){
			private static final long serialVersionUID = 1L;

				public String getToolTipText(MouseEvent evt) { 
        			int index = locationToIndex(evt.getPoint()); 
        			ChannelThread item = (ChannelThread)getModel().getElementAt(index); 
        			
        			StringBuilder sb = new StringBuilder();
        			
        			sb.append(item.getChannelName()).append(" playlist:\n");
        			
        			if (item.getLoopMusic() != null) {
        				for (SoundFile soundFile: item.getLoopMusic().getSoundFiles()) {
        					sb.append("\n ").append(soundFile.getFileName());
        				}
        			}
        			
        			return sb.toString(); 
        		}
				
        }; 
	
        JScrollPane channelListScroller = new JScrollPane(channelList);
        currentlyPlayingPanel.add(channelListScroller);

        tabPane.addTab("Currently Playing", currentlyPlayingPanel);
        
        JPanel configurationPanel = new JPanel();
        configurationPanel.setLayout(new BorderLayout());
        
        JTree soundsTree = new JTree(new ConfigurationXMLTreeAdapter(sp.getSoundsXML()));
        
        JScrollPane soundsTreeScroller = new JScrollPane(soundsTree);
        configurationPanel.add(soundsTreeScroller, BorderLayout.CENTER);
        
        final JPanel soundInfoPanel = new JPanel();
        soundInfoPanel.setLayout(new GridLayout(0,2));

        soundInfoPanel.add(new Label("Pattern:"));
        final JLabel soundInfoPatternLabel = new JLabel();
        soundInfoPanel.add(soundInfoPatternLabel);
        
        soundInfoPanel.add(new Label("Channel:"));
        final JLabel soundInfoChannelLabel = new JLabel();
        soundInfoPanel.add(soundInfoChannelLabel);
        
        soundInfoPanel.add(new Label("Loop:"));
        final JLabel soundInfoLoopLabel = new JLabel();
        soundInfoPanel.add(soundInfoLoopLabel);
        
        soundInfoPanel.add(new Label("Timeout:"));
        final JLabel soundInfoTimeoutLabel = new JLabel();
        soundInfoPanel.add(soundInfoTimeoutLabel);
        
        soundInfoPanel.add(new Label("Concurency:"));
        final JLabel soundInfoConcurencyLabel = new JLabel();
        soundInfoPanel.add(soundInfoConcurencyLabel);
        
        configurationPanel.add(soundInfoPanel, BorderLayout.PAGE_END);
        soundInfoPanel.setVisible(false);
        
        
        soundsTree.addTreeSelectionListener(new TreeSelectionListener() {
        	@Override
        	public void valueChanged(TreeSelectionEvent e) {
        		
        		Object selectedItem = e.getNewLeadSelectionPath().getLastPathComponent();
        		
        		if (selectedItem instanceof Sound) {
        			Sound sound = (Sound) selectedItem;
        			
        			soundInfoPanel.setVisible(true);
        			soundInfoPatternLabel.setText(sound.getLogPattern());
        			soundInfoChannelLabel.setText(sound.getChannel());
        			soundInfoLoopLabel.setText(sound.getLoop()==null?"":sound.getLoop().toString());
        			soundInfoTimeoutLabel.setText(sound.getTimeout()==null?"no timeout":sound.getTimeout()+"ms");
        			soundInfoConcurencyLabel.setText(sound.getConcurency()==null?"no concurency":sound.getConcurency().toString());
        			
        		} else if (selectedItem instanceof SoundFile) {
        			
        			soundInfoPanel.setVisible(false);
        			
        		} else if (selectedItem instanceof String) {
        			
        			soundInfoPanel.setVisible(false);
        			
        		}
        	}
        });

        tabPane.addTab("Sounds Configuration", configurationPanel);

	}
	
    private static void createAndShowGUI(final ConfigurationXML config, SoundProcesor sp) {
        //Create and set up the window.
        JFrame frame = new JFrame("SoundSense");
        frame.setIconImage(Toolkit.getDefaultToolkit().getImage("icon.png"));
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        frame.addWindowListener(new WindowListener() {
			
			@Override
			public void windowClosed(WindowEvent arg0) {
			}

			@Override
			public void windowActivated(WindowEvent arg0) {
			}

			@Override
			public void windowClosing(WindowEvent arg0) {
				config.saveConfiguration();
			}

			@Override
			public void windowDeactivated(WindowEvent arg0) {
			}

			@Override
			public void windowDeiconified(WindowEvent arg0) {
			}

			@Override
			public void windowIconified(WindowEvent arg0) {
			}

			@Override
			public void windowOpened(WindowEvent arg0) {
			}

		});
        
        Gui gui = new Gui(config, sp);
                
        //Add content to the window.
        frame.add(gui, BorderLayout.CENTER);

        //Display the window.
        frame.pack();
        frame.setVisible(true); 
    }

    public static void newGui(final ConfigurationXML config, final SoundProcesor sp) {
	    SwingUtilities.invokeLater(new Runnable() {
	        public void run() {
	            createAndShowGUI(config, sp);
	        }
	    });
    }

}
